#=======================================================================================================
# PACKAGE NAME: Clustering
#    FUNCTIONS:	 - TieInputFastaFile (Read and create needed data from a Fasta file without memory),
#
# REQUIREMENTS:  Perl language only
#       AUTHOR:  Sebastien TERRAT <sebastien.terrat@dijon.inra.fr>
#      COMPANY:  GenoSol platform (UMR 1347 INRA/uB/AgroSup)
#      VERSION:  0.0
#    REALEASED:  18/05/2011
#     REVISION:  XXX
#=======================================================================================================

# Name of the package defined for the main program
package PerlLib::NotUsed;

# Needed libraries
use strict;
use warnings;

# Library needed for the analysis and the treatment of the initial file without storing it in memory.
# Each element in the array corresponds to a record in the file, not the stored file itself.
use Tie::File;

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Function dedicated to record all data from the file without storing it in memory. This permits to 
# treat huge amounts of data such as Iluumina data. This function takes as input the filehandle already
# opened using a previous function. Then, it gives back as output three references (the array containing
# data from the function Tie::File, the hash table with IDs of reads and the positions in the first 
# array of the sequences associated, and finally an array containing the IDs of the sequences).
sub TieInputFastaFile
{
	my $file = $_[0];
	my $i = 0;
	my @file_data;
	my @data = ();
	my %data = ();
	
	use Fcntl 'O_RDONLY';
	tie @file_data, 'Tie::File', \*$file or die "==> Can't open $file\n <==", memory => 0 ;
	
	for($i=0;$i<=$#file_data;$i++)
	{
		#We must avoid empty lines to avoid a problem during the analysis due to the shuffling of reads
		if ($file_data[$i] eq "\n") { next; }
		
		#If the reading line starts with the character ">"
		if ($file_data[$i] =~ m/^>/)
		{
			#Store the sequence ID and the line (using $.) where we can found the sequence itself
			$data{$file_data[$i]} = $i + 1;
			#Store the sequence ID also in an array (@data) for further use
			push(@data, $file_data[$i]);
		}
	}
	return (\@file_data, \@data, \%data);
}
# End of the function
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



