#=======================================================================================================
# PACKAGE NAME: ProgramLaunchers
#    FUNCTIONS:	 - ComputationLauncher (Launch the COMPUTATION step, rax and/or clean)
#				 - GlobAnalysisLauncher (Launch the GLOBAL_ANALYSIS step).
#
#	   DETAILS:  1.1 (17/09/2014): We moved the DeMultiplexingLauncher function into the DeMultiplexing 
#				 library to regroup all needed functions for this particular step.
#
#				 1.2 (11/12/2015): Deletion of Function called RandomLauncher and transfer this part 
#				 into the RandomSelection library to regroup all needed functions and programs for this
#				 particular step.
#
# REQUIREMENTS:  Perl language only
#       AUTHOR:  Sebastien TERRAT <sebastien.terrat@dijon.inra.fr>
#      COMPANY:  UMR 1347 Agroecology - Platform GenoSol
#      VERSION:  1.2
#    REALEASED:  17/04/2014
#     REVISION:  11/12/2015
#=======================================================================================================

# Name of the package defined for the main program
package PerlLib::ProgramLaunchers;
# Needed libraries
use strict;
use warnings;
use FindBin;
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Function needed to launch the computation step chosen by the user and subsequent treatment steps.
sub ComputationLauncher
{
	#Needed empty variables
	my @files_Clust = ();
	my $file = "";
	my $file_name_pipe = "";
	my @parameters = @{$_[2]};
	
	#Open the corresponding directory containing all files needed to be treated to launch one treatment by file
	opendir (DIR, "Result_files/$_[1]/") || die "can't opendir Result_files/$_[1]/: $!";
	#Loop to read all files found in the defined directory chosen by the user
	LOOP_COMPUT:while (defined($file = readdir(DIR)))
	{
		#No treatment for files '.' and '..'
	  	if ($file =~ m/^\.\.?$/){next LOOP_COMPUT;}
	  	elsif (-d $file){next LOOP_COMPUT;}
	  	else {push(@files_Clust, $file);}
	}
	closedir (DIR);
	@files_Clust = sort(@files_Clust);	
	foreach $file (@files_Clust)
  	{
		#Print for the user information that the program is started
		PerlLib::Miscellaneous::PrintScreen("Computation_rc_estimators_3.0.py",$file,"started");
		#Needed command line constructed. Launch the command line instruction using the "`". And we catch 
		#the print in the $file_name_pipe variable to be used further
		$file_name_pipe = `python $FindBin::RealBin/Prog_Python/Computation_rc_estimators_3.0.py $_[0] $parameters[2] $parameters[3] $parameters[4] $parameters[5] $parameters[6] $parameters[7] $_[1] $file`;
		#Print for the user information that the program is terminated
		PerlLib::Miscellaneous::PrintScreen("Computation_rc_estimators_3.0.py",$file,"terminated");
	}
	#Print for the user information that the program is started
	PerlLib::Miscellaneous::PrintScreen("Curves_drawing_1.7.py","$_[0] files","started");
	#Needed command line constructed. Launch the command line instruction using the "`". And we catch 
	#the print in the $file_name_pipe variable to be used further if needed
	$file_name_pipe = `python $FindBin::RealBin/Prog_Python/Curves_drawing_1.7.py $_[0] $parameters[2] $parameters[3] $parameters[4]`;
	#Print for the user information that the program is terminated
	PerlLib::Miscellaneous::PrintScreen("Curves_drawing_1.7.py","$_[0] files","terminated");
}
# End of the function
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Function needed to launch the global analysis of samples chosen by the user with subsequent 
# treatment steps.
sub GlobAnalysisLauncher
{
	#Needed empty variable
	my $file_name_pipe = "";
	#Print for the user information that the program is started
	PerlLib::Miscellaneous::PrintScreen("Globanalysis_matrix_computation_2.8.pl","Concatenated_reads.fasta.fasta","started");
	#Needed command line constructed. Launch the command line instruction using the "`". And we catch 
	#the print in the $file_name_pipe variable to be used further if needed
	$file_name_pipe = `perl $FindBin::RealBin/Prog_Perl/Globanalysis_matrix_computation_2.8.pl $_[0] $_[1] $_[2] $_[3] $_[4] $_[5] $_[6] $_[7] $_[8]`;
	#Print for the user information that the program is terminated
	PerlLib::Miscellaneous::PrintScreen("Globanalysis_matrix_computation_2.8.pl","Concatenated_reads.fasta.fasta","terminated");
}
# End of the function
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
1;
