#=======================================================================================================
#         FILE:  Filter_reads_by_length_1.0.pl
#        USAGE:  perl Filter_reads_by_length_1.0.pl
#  DESCRIPTION:  
# REQUIREMENTS:  Perl language only
#       AUTHOR:  Sebastien TERRAT <sebastien.terrat@dijon.inra.fr>
#      COMPANY:  GenoSol platform (UMR 1347 INRA/uB/AgroSup)
#      VERSION:  XXX
#    REALEASED:  
#     REVISION:  
#=======================================================================================================
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#!/usr/bin/perl

#Print errors and warnings during the implementation of the perl program
use strict;
use warnings;

#Needed homemade modules stored in the PerlLib library
use PerlLib::Miscellaneous;

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Function dedicated to catch and store all file names found in a defined directory containing only
# files.
sub ListFilesInFolder
{
	#Needed variables
	my $file_name = "";
	my @file_names = ();
	my $folder = $_[0];
	
	#Open the directory containing all alignment files for all independant samples
	opendir (DIR, "$folder/") || die "can't opendir $folder: $!";
	#For each file found in the directory Raw_data/
	LOOP:while (defined($file_name = readdir(DIR)))
	{
		#No treatment for files '.' and '..'
	  	if ($file_name =~ m/^\.\.?$/){ next LOOP; }
	  	#We store their names in the align_file_names array
	  	else { push(@file_names, $file_name); }
	}
	#We then close the directory
	closedir DIR;
	return \@file_names;
}
# End of the function
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#=======================================================================================================
#=======================================================================================================
# MAIN PROGRAM
#=======================================================================================================
#=======================================================================================================

my @file_names = ();
my $ref_array = "";
my $file = "";

my $len_threshold = 500;

my $length = 0;
my $nb_tot_reads = 0;
my $nb_kept_reads = 0;
my $nb_deleted_reads = 0;
my $ID = "";
my $seq = "";

$ref_array = &ListFilesInFolder("Raw_data/Good");
@file_names = @{$ref_array};

open(F_SUM, ">Result_files/LENGTH_FILTER/Summary.txt")|| die "==> File Not Found: Summary.txt<==";
printf(F_SUM "Filter by length of reads (deletion if read length higher than 500 bases)\n");
printf(F_SUM "File name\tNb of treated reads\tNb of kept reads\tNb of deleted reads\n");
foreach $file (@file_names)
{
	open(F_FASTA, "Raw_data/Good/$file")|| die "==> File Not Found: $file<==";
	open(F_KEPT, ">Result_files/LENGTH_FILTER/GOOD/$file")|| die "==> File Not Found: $file<==";
	open(F_DEL, ">Result_files/LENGTH_FILTER/BAD/$file")|| die "==> File Not Found: $file<==";
	while(<F_FASTA>)
	{
		$ID = $_;
		if($_ =~ m/_(\d*)_length\=(\d*)\n/)
		{
			$nb_tot_reads = $nb_tot_reads + $1;
			if($2 >= $len_threshold)
			{
				printf(F_DEL $_);
				$nb_deleted_reads = $nb_deleted_reads + $1;
				$seq = <F_FASTA>;
				printf(F_DEL $seq);
			}
			else
			{
				printf(F_KEPT $_);
				$nb_kept_reads = $nb_kept_reads + $1;
				$seq = <F_FASTA>;
				printf(F_KEPT $seq);
			}
		}
	}
	close(F_FASTA);
	close(F_KEPT);
	close(F_DEL);
	printf(F_SUM "$file\t$nb_tot_reads\t$nb_kept_reads\t$nb_deleted_reads\n");
	$nb_tot_reads = 0;
	$nb_kept_reads = 0;
	$nb_deleted_reads = 0;
}
close(F_SUM);
exit(0);


