#=======================================================================================================
#         FILE:  Merge_alignments_1.0.pl
#        USAGE:  perl Merge_alignments_1.0.pl
#  DESCRIPTION:  
# REQUIREMENTS:  Perl language only
#       AUTHOR:  Sebastien TERRAT <sebastien.terrat@dijon.inra.fr>
#      COMPANY:  GenoSol platform (UMR 1347 INRA/uB/AgroSup)
#      VERSION:  XXX
#    REALEASED:  
#     REVISION:  
#=======================================================================================================
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#!/usr/bin/perl

#Print errors and warnings during the implementation of the perl program
use strict;
use warnings;

#Needed homemade modules stored in the PerlLib library
use PerlLib::ManagingDirectories;
use PerlLib::ManagingGlobalAnalysis;
use PerlLib::ProgramLaunchers;
use PerlLib::SummaryFilesDefinition;
use PerlLib::VerifyingUserChoices;
use PerlLib::Miscellaneous;
use PerlLib::ManagingFormatFiles;
use PerlLib::ManagingQuality;
use PerlLib::Preprocessing;
use PerlLib::Dereplication;
use PerlLib::GlobalAlignment;
use PerlLib::PoolingReads;
use PerlLib::CheckInputFile;
use PerlLib::Clustering;


#Needed variables
my $file = "";
my @files = ();
my $folder_align = "Result_files/CLEAN_INFERNAL_ALIGNMENT";
my %seq_cons = ();
my $len = 0;
my $len_max = 0;
my ($i,$j,$k) = (0,0,0);
my $chosen_file_cons = "";
my $ref_cons = "";
my $ref_cons2 = "";
my @ref_cons = ();
my @treated_cons = ();
my %patterns = ();

my $added_gaps = "";
my $pos_added_gaps = -1;
my @pattern = ();
my @treated_read = ();
my $element = "";
my $adds = 0;
my @tmp = ();
my @tmp2 = ();
my @tmp3 = ();

my @splice = ();

my $ref_array = "";
my %ID_align_names = ();
my %ID_reads_file = ();
my $ID_file = "";
my $test = 0;

my $GC_RF_seq = "";
my $GC_SS_seq = "";
my $GC_test = 0;

my $eval = "";

#=======================================================================================================
# 01: Here, we catch and organize all consensus sequences from all treated files.
#=======================================================================================================
open(F_SUM, ">Test.txt") || die "can't open the summary file";
print("STARTED\t".localtime(time)."\n");
#We open the needed directory
opendir (DIR, "$folder_align") || die "can't open the alignment directory: $!";
#And for each file in the directory
LOOP01:while (defined($file = readdir(DIR)))
{
	if ($file =~ m/^\.\.?$/) { next LOOP01; }
	else
	{
		#We open each file
		push(@files, $file);
		open(F_IN, "$folder_align/$file")|| die "==> File Not Found: $folder_align/$file<==";
		#For each line in the file
		while(<F_IN>)
		{
			#We search the line corresponding to the ID of the consensus sequence
			if ($_ =~ m/\>\#\=GC_RF/)
			{
				#We also catch the consensus sequence itself
				$seq_cons{$file} = <F_IN>;
				#We store its length
				$len = length($seq_cons{$file});
				#We keep only the longest consensus sequence from all files
				if($len_max < $len)
				{
					#We catch the longest consensus sequence 
					$ref_cons = $seq_cons{$file};
					#We also store the name of the file and the length of the sequence
					$chosen_file_cons = $file;
					$len_max = $len;
				}
			}
		}
		close(F_IN);
	}
}
closedir(DIR);

#=======================================================================================================
# 02: We create the longest consensus sequence merging all information from all independent alignments
#=======================================================================================================

#First, we store the ref_cons string in an array
@ref_cons = split("", uc($ref_cons));
#For each alignment file that we have to treat
LOOP02:foreach $file (keys(%seq_cons))
{
	#We avoid the treatment of the file used initially to select the longest consensus sequence
	if($file eq $chosen_file_cons) { next LOOP02; }
	#We also store the consensus sequence of the treated alignement in an array
	@treated_cons = split("", uc($seq_cons{$file}));
	#Loop to read each character and all characters of all alignments
	LOOP03:for($i=0;$i<$#ref_cons;$i++)
	{
		LOOP04:for($j=0;$j<$#treated_cons;$j++)
		{
			#If the same characters are found in the two alignments
			if($ref_cons[$i] eq $treated_cons[$j])
			{
				#We store the character and then we go to the next one
				$ref_cons2 .= $ref_cons[$i];
				$i++;
			}
			#If the two characters are different
			else
			{
				#We check which consensus sequence add a gap here to add a gap to the other
				if($ref_cons[$i] eq ".")
				{
					#We store the character of the longest consensus sequence and then we go to the 
					#next one, staying in the good character of the treated consensus
					$ref_cons2 .= $ref_cons[$i];
					$i++;
					$j--;
				}
				#We have to add a gap to the reference consensus alignment
				else { $ref_cons2 .= "."; }
			}
		}
		if($j == $#treated_cons)
		{
			@splice = splice(@ref_cons, $i);
			$ref_cons2 .= join("", @splice);
			#Finally we store the global reference consensus in the array @ref_cons and in a variable
			$GC_RF_seq = $ref_cons2;
			@ref_cons = split("", uc($ref_cons2));
			$ref_cons2 = "";
			next LOOP02;
		}
	}
	#Finally we store the global reference consensus in the array @ref_cons and in a variable
	$GC_RF_seq = $ref_cons2;
	@ref_cons = split("", uc($ref_cons2));
	$ref_cons2 = "";
	next LOOP02;
}
close(F_OUT);

#=======================================================================================================
# 03: We then create each pattern for all treated files to add efficiently all gaps.
#=======================================================================================================

#For each alignment file that we have to treat
LOOP05:foreach $file (keys(%seq_cons))
{
	#We store the consensus sequence of the treated alignement in an array
	@treated_cons = split("", uc($seq_cons{$file}));
	#Loop to read each character and all characters of all alignments
	LOOP06:for($i=0;$i<$#ref_cons;$i++)
	{
		LOOP07:for($j=0;$j<$#treated_cons;$j++)
		{
			#If the same characters are found in the two alignments
			if($ref_cons[$i] eq $treated_cons[$j])
			{
				#If the area of gaps is over (as we do not add another gap), we then
				#store all needed data in the @pattern array and erase all not needed 
				#data and variables.
				if($pos_added_gaps > -1)
				{
					push(@pattern, "$pos_added_gaps"."_".$added_gaps);
					$pos_added_gaps = -1;
					$added_gaps = "";
				}
				$i++;
			}
			#If the two characters are different
			else
			{
				if($ref_cons[$i] eq ".")
				{
					#Here, we evaluate if it's a new area of gaps or an actual one
					if($pos_added_gaps == -1)
					{
						#If it's a new one, we keep the start position in the treated read
						#and we start also the $added_gaps variable
						$added_gaps = "\.";
						#We also check if there is already a gap opened before to increase
						#the gap instead of opening a new one. To do this, we check the 
						#length of the gap using a while loop.
						$k = -1;
						while($treated_cons[$j + $k] eq ".") { $k--; }
						#We then define the center of the found gap
						$k = int($k/2 + 0.5);
						#And we add to the position where the gap will be added this new value
						$pos_added_gaps = $j + $k;
					}
					#If it's gap area already started, we continue it adding a new gap
					else { $added_gaps .= "\."; }
					$i++;
					$j--;
				}
				else
				{
					#Test to evaluate if it's a new area of gaps or an actual one
					if($pos_added_gaps > -1)
					{
						#If the area of gaps is over (as we do not add another gap), we then
						#store all needed data in the @pattern array and erase all not needed 
						#data and variables.
						push(@pattern, "$pos_added_gaps"."_".$added_gaps);
						$pos_added_gaps = -1;
						$added_gaps = "";
					}
				}
			}
		}
	}
	$patterns{$file} = [@pattern];
	@pattern = ();
}

#=======================================================================================================
# 04
#=======================================================================================================


#Here, we catch and associate the IDs given by the global alignment to file names
$ref_array = PerlLib::GlobalAlignment::SearchIDsFileNames(\@files);
%ID_align_names = %{$ref_array};
open(F_OUT, ">$folder_align"."/Align_Derep_Concatenated_reads.fasta.fasta")|| die "==> File Not Found: Can't create file<==";
#For each found alignment file
foreach $ID_file (keys(%ID_align_names))
{
	#We open first the file containing all merge reads for all samples
	open(F_GLOB, "Result_files/RANDOM_CLEANED/Derep_Concatenated_reads.fasta.fasta")|| die "==> File Not Found: Derep_Concatenated_reads.fasta.fasta<==";
	#Here, we catch all read IDs for a given alignment file to use them later
	while(<F_GLOB>) { if($_ =~ m/^>$ID_file\|/) { $ID_reads_file{$_} = $_; } }
	close(F_GLOB);
	#Then, we open the file containing details about the dereplication of reads for all samples
	open(F_DEREP, "Summary_files/Global_Analysis/Derep_details_Concatenated_reads.fasta.fasta")|| die "==> File Not Found: Derep_details_Concatenated_reads.fasta.fasta<==";
	#We then search for dereplicated reads in the global file to find the corresponding ID 
	#in the independent alignment file.
	while(<F_DEREP>)
	{
		@tmp = split("\t", $_);
		#If the ID file is found, it's a dereplicated read between several files
		if(exists($ID_reads_file{">".$tmp[0]."\n"}))
		{
			@tmp2 = split(" ", $tmp[1]);
			$test = 1;
			for($i=1;$i<=$#tmp2;$i++)
			{
				if($tmp2[0] eq $tmp2[$i]) { $test++; }
			}			
			#TEST egalité doublons dans la ligne !!
			if($test == 1)
			{
				#So, we keep the original ID as value and the new ID as key to efficiently treat the file
				$ID_reads_file{">".$tmp2[0]."\n"} = ">".$tmp[0]."\n";
				delete($ID_reads_file{">".$tmp[0]."\n"});
			}
			else
			{
				@tmp3 = split("_", $tmp2[0]);
				delete($ID_reads_file{">".$tmp[0]."\n"});		
				$ID_reads_file{">".$tmp3[0]."_".$test."_".$tmp3[2]."\n"} = ">".$tmp[0]."\n";
			}
		}
	}
	close(F_DEREP);
	#Here, we catch the needed pattern to modify the aligned reads.
	$file = $ID_align_names{$ID_file};
	@pattern = @{$patterns{$file}};
	#We then open the corresponding alignment file
	open(F_IN, "$folder_align/$file")|| die "==> File Not Found: $folder_align/$file<==";
	printf(F_SUM "$file\n");
	#For each line in the file
	while(<F_IN>)
	{
		#If the line is a descriptive line, we just keep it in the next file
		if($_ =~ s/^>/\>$ID_file\|/)
		{
			if(exists($ID_reads_file{$_}))
			{
				printf(F_OUT $ID_reads_file{$_});
				delete($ID_reads_file{$_});
				$_ = <F_IN>;
				#We first split the sequence and store it in an array
				@treated_read = split("", $_);
				#Then, we use the defined pattern, using each element
				foreach $element (@pattern)
				{
					#We first catch the position and the gaps
					@tmp = split("_", $element);
					#We add gaps in the array using the splice function
					splice(@treated_read, $tmp[0]+$adds, 0, $tmp[1]);
					#We count the number of adds done
					$adds++;
				}
				#We finally reinitialize the needed variables
				printf(F_OUT join("",@treated_read));
				$adds = 0;
			}
		}
		if(($_ =~ m/GC_SS_cons/)&&($GC_test == 0))
		{
			#Here, we catch the GC_SS_cons secondary structure sequence
			$GC_SS_seq = <F_IN>;
			#We first split the sequence and store it in an array
			@treated_read = split("", $GC_SS_seq);
			#Then, we use the defined pattern, using each element
			foreach $element (@pattern)
			{
				#We first catch the position and the gaps
				@tmp = split("_", $element);
				#We add gaps in the array using the splice function
				splice(@treated_read, $tmp[0]+$adds, 0, $tmp[1]);
				#We count the number of adds done
				$adds++;
			}
			$adds = 0;
			$GC_SS_seq = join("",@treated_read);
			$GC_test++;
		}
	}
	close(F_IN);
	%ID_reads_file = ();
}

#We finally print the GC_RF data to recreate an Infernal alignment
printf(F_OUT ">#=GC_SS_cons\n".$GC_SS_seq.">#=GC_RF\n".$GC_RF_seq);
close(F_OUT);
print("TERMINATED\t".localtime(time)."\n");
close(F_SUM);
exit(0);
