#=======================================================================================================
#         FILE:  Computation_rc_estimators_2.5.py
#        USAGE:  python Computation_rc_estimators_2.5.py
#  DESCRIPTION:  To use this program, you first need cluster files (one or more) stored in the 'IN/' 
#		 directory. This tool then will take each cluster file, and treat them to calculate a
#		 rarefaction curve based on the formula defined by several authors. For each cluster,
#		 a maximum of 1000 points are determined, and results are stored in 'OUT/' directory
#		 in a new file named : "rc_cutoff_inputfilename.clust". This tool treat all files found,
#		 and need only the defined cutoff by the user.
#                This tool is also dedicated specifically to be used in a server efficiently. Indeed, 
#		 for each step of calculation for the rarefaction curve, a cpu is used. This is very fast
#		 for small sets of sequences, and fastly enough for large sets of sequences.
#
#                2.5 Modifs (30/08/2012): We modify the program to use parameters givn as input by the 
#                main program to define what steps are done and, what not, as chosen by the user.
#
#                3.0 Modifs (06/11/2012): We modify and clarify the Python program using the Functions.py
#                as some parts of the program was already implemented in ohter systems ans functions, e.g.
#                the percent computing. Moreover, some parts of the program have also been simplifed, 
#                modified and optimized in specific functions for the program itself (like the 
#                rarefaction_computation function for example).
#
# REQUIREMENTS:  Python language only (version 2.7)
#       AUTHOR:  Sebastien TERRAT <sebastien.terrat@dijon.inra.fr>
#      COMPANY:  GenoSol Platform
#      VERSION:  3.0
#    REALEASED:  24/01/2011
#     REVISION:  06/11/2012
#=======================================================================================================
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#=======================================================================================================

#Import needed modules (needed : comb for gmpy, corresponding at combinatory calculations)
import Library

import math
import decimal
import gmpy
import os
import sys



decimal.getcontext().prec = 6

#Defined variables for the main program and for the calculation of the rarefaction curve
treated_files = list()
Result_tmp = decimal.Decimal(0)
Result2 = ""
x,result = 0,0
Rarefact_chose = ""
string_printed = "Sample\t"
tmp_printed = list()
Result_printed = [""]
Children = []
Child_pid = 0

#-------------------------------------------------------------------------------------------------------
#2.5: First part of the program, catching data chosen by the user to analyze files in the directory
#and also estimators needed to be calculated and chose by the user
#-------------------------------------------------------------------------------------------------------
#Asking for the desired threshold of dissimilarity
cutoff = sys.argv[2]
cutoff = float(cutoff)
cutoff = cutoff/100
#1.5: Catching user choices given by the main program
User_choice = sys.argv[1]
Rarefact_chose = sys.argv[3]
Rank_abundance_chose = sys.argv[4]
Chao_chose = sys.argv[5]
ACE_chose = sys.argv[6]
Index_chose = sys.argv[7]
Folder_name = sys.argv[8]
treated_file = sys.argv[9]
Results = ""

#Defined variables for chao calculation
C_chao1,S_chao1,var_S_chao1 = 0,0,0
n1,n2 = 0,0
LCI95_chao1,UCI95_chao1 = 0,0
#Defined variables for ACE calculation
N_rare_ACE,C_ACE = 0,0
Gamma_ACE,S_rare_ACE = 0,0
S_ACE,S_abund_ACE,n1_ACE = 0,0,0
#Defined variables for shannon and simpson index calculations
S_bootstrap = 0
H_Shannon,var_H_Shannon, Evenness = decimal.Decimal(0),decimal.Decimal(0),decimal.Decimal(0)
D_Simpson = decimal.Decimal(0)

#-------------------------------------------------------------------------------------------------------
#Needed functions
#-------------------------------------------------------------------------------------------------------
def combinatory(N,n): ##combinatory (permit the calculation of combinatorial values)
    X = math.factorial(N)
    Xb = math.factorial(n)
    Xc = math.factorial(N-n)
    final = X/(Xb*Xc)
    return final

def rank_abundance_computation(Clusters, User_choice, cutoff, treated_file):
    #Needed variable
    j = 1
    #Computation to determine the relative abundance of each defined OTU
    Rank_abundance_data = Library.Functions.compute_percents(Clusters)
    #We then reverse it to organize data by descending order
    Rank_abundance_data.reverse()
    #We open the fils hat will contain computed data
    Rank_abundance_curve = open("Summary_files/Curves_Data/"+"Rank-abundance_curve_"+ User_choice +"_"+str(cutoff)+"_"+treated_file, 'w')
    #Then, we create a loop to create relative abundance of each cluster compared to the total
    for value in Rank_abundance_data:
        Rank_abundance_curve.write(str(j) + "\t"+ str(value) + "\n")
        j += 1
    Rank_abundance_curve.close()

def rarefaction_computation(Matrix, User_choice, cutoff, treated_file, Nb_seqs_tot, Nb_clusters):
    #Needed empty variables
    n = 0
    N_Ni = 0
    Result = float()
    Result_fin = float()
    #2.5: Create a new file to store the rarefaction curve calculation data
    Rarefact_curve = open("Summary_files/Curves_Data/"+"Rarefactcurve_"+ User_choice +"_"+str(cutoff)+"_"+treated_file, 'w')
    Rarefact_curve.close()
    #A maximum of 1000 points are determined, verification of the number of sequences
    #If the number of sequences if less than 1000, we treat each point
    if Nb_seqs_tot < 1000:
        Nb_steps_calc = 1
    #Else, we calculate the needed step to do 1000 points
    else:
        Nb_steps_calc = math.floor(Nb_seqs_tot/1000)
    #Loop to calculate the rarefaction curve
    while n < Nb_seqs_tot:
        Child_pid = os.fork()
        if Child_pid == 0:
            #Calculation of the rarefaction curve points (details in the formula)
            for element in Matrix:
                #For each element in the matrix, we calculate N_Ni
                N_Ni = Nb_seqs_tot - element[0]
                #Sum of combinatory values in the upper formula using N_Ni, n and Nb_seqs_tot
                Result = Result + element[1]*gmpy.comb(gmpy.mpz(N_Ni),gmpy.mpz(n))
                #Determination of the value of the rarefaction curve for n sequences in Nb_seqs_tot data
                Result_fin = float(Nb_clusters) - Result/(gmpy.comb(gmpy.mpz(Nb_seqs_tot),gmpy.mpz(n)))
            #In the result file, storage of results for this step
            Result2 = str(n) + "\t" + str(Result_fin) + "\n"
            #Append the file to store results
            Rarefact_curve = open("Summary_files/Curves_Data/"+"Rarefactcurve_"+ User_choice +"_"+str(cutoff)+"_"+treated_file, 'a')
            Rarefact_curve.write(Result2)
            Rarefact_curve.close()
            os._exit(0)
        else:
            Children.append(Child_pid)
            #Increment n using the step calculated before
            n += Nb_steps_calc
    #Wait for all child processes to continue the program
    for Child_pid in Children:
        status = os.waitpid(Child_pid,0)


#-------------------------------------------------------------------------------------------------------
#Main program : selection and storage of the chosen cluster
#-------------------------------------------------------------------------------------------------------

#Open the targeted cluster file 
Cluster_file = open("Result_files/"+Folder_name+"/"+treated_file, 'r')
#For each read line of the file (as the line is not empty)
Read_lines = Cluster_file.readlines()
#We then close the file
Cluster_file.close()

#We use the function to found and catch all needed data (nb_seqs_tot, nb_clusters and the details of OTUs)
(Nb_seqs_tot, Nb_clusters, Cluster_analyzed) = Library.Functions.sort_chosen_cluster_details(Read_lines, cutoff)

#We compute here the Rank_abundance curve based on examples found in various publications
if Rank_abundance_chose.lower() == "yes":
    if ("Concatenated_reads.fasta.fasta" not in treated_file):
        rank_abundance_computation(Cluster_analyzed, User_choice, cutoff, treated_file)

#Here, we used the needed function to create the needed matrices to determine rarefaction curves and all indexes.
Matrix = Library.Functions.computing_matrix_OTUs(Cluster_analyzed)

#Computation of the rarefaction curve based on the formula found in various publications
if Rarefact_chose.lower() == "yes":
    if ("Concatenated_reads.fasta.fasta" not in treated_file):
        rarefaction_computation(Matrix, User_choice, cutoff, treated_file, Nb_seqs_tot, Nb_clusters)
    
#-------------------------------------------------------------------------------------------------------
#Calculation of the chao estimator based on the formula found in various publications if wanted
#for details : see Appendix.A
#-------------------------------------------------------------------------------------------------------

Results+= str(treated_file)+"\t"+str(Nb_seqs_tot)+"\t"+str(Nb_clusters)+"\t"
#Verification for the choice of the user for calculation of the chao1 richness estimator
if Chao_chose.lower() == "yes":
    #We search in the matrix the number of singletons (n1) and doubletons (n2)
    for element in Matrix:
        if element[0] == 1:
            n1 = element[1]
        if element[0] == 2:
            n2 = element[1]
    #Calculation of S_chao1 based on n1 and n2 values
    if n1>0 and n2 >= 0 or n1==0 and n2==0:
        S_chao1 = Nb_clusters + (n1*(n1-1))/(2*(n2+1))
    if n1==0 and n2 >= 0:
        S_chao1 = Nb_clusters + (n1*n1)/(2*n2)
    #Calculation of var_S_chao1 based on n1 and n2 values
    if n1>0 and n2>0:
        #Bias corrected formula
        var_S_chao1 = (n1*(n1-1))/(2*(n2+1))+(n1*(2*n1-1)*(2*n1-1))/(4*(n2+1)*(n2+1))+\
            (n1*n1*n2*(n1-1)*(n1-1))/(4*(n2+1)*(n2+1)*(n2+1)*(n2+1))
        #Classic formula
        #var_S_chao1 = n2*(0.5*math.pow(n1/n2,2)+math.pow(n1/n2,3)+0.25*math.pow(n1/n2,4))
    #Calculation of var_S_chao based on the numbers n1 and n2 and also on experimentally values
    if n1>0 and n2==0:
        var_S_chao1 = ((n1*(n1-1))/(n1*(2*n1-1))*((2*n1-1))/4)-(n1*n1*n1*n1/(4*S_chao1))
    if n1==0 and n2>0:
        var_S_chao1 = Nb_clusters*math.exp((-Nb_seqs_tot)/Nb_clusters)*\
            (1-math.exp((0-Nb_seqs_tot)/Nb_clusters))
    if S_chao1 == Nb_clusters:
        S_chao1+=1
    #Calculation of C_chao1 based on previous data calculated
    Result_tmp = math.log((1 + (var_S_chao1)/math.pow(S_chao1-Nb_clusters,2)),math.e)
    C_chao1 = math.exp((1.96*math.sqrt(Result_tmp)))
    #Calculation of LCI95_chao1 and UCI95_chao1 based on previous data calculated
    LCI95_chao1 = Nb_clusters + (S_chao1-Nb_clusters)/C_chao1
    UCI95_chao1 = Nb_clusters + (S_chao1-Nb_clusters)*C_chao1
    Result_tmp = 0
    Results+=str(round(S_chao1,3))+"\t"+str(round(var_S_chao1,3))\
                    +"\t"+str(round(LCI95_chao1,3))+"\t"+str(round(UCI95_chao1,3))+"\t"

#-------------------------------------------------------------------------------------------------------
#Calculation of the ACE richness estimator based on the formula found in various publications if wanted
#for details : see Appendix.A
#-------------------------------------------------------------------------------------------------------
#Verification for the choice of the user for calculation of the ACE richness estimator
if ACE_chose.lower() == "yes":
    #First we need to calculate S_rare_ACE and S_abund_ACE based on the matrix
    #In the same time, we determine N_rare_ACE based on the number of OTUs with i individuals
    for element in Matrix:
        if element[0] <= 10:
            S_rare_ACE += element[1]
            N_rare_ACE += (element[0]*element[1])
        S_abund_ACE = Nb_clusters - S_rare_ACE
        if element[0] == 1:
            n1_ACE = element[1]
    #Determination of C_ACE based on previous calculations
    C_ACE = decimal.Decimal(1) - (decimal.Decimal(n1_ACE)/decimal.Decimal(N_rare_ACE))
    #Determination of Gamma_ACE based on previous calculations
    for element in Matrix:
        if element[0] <= 10:
            Result_tmp += decimal.Decimal(element[0])*decimal.Decimal((element[0]-1))*\
                          decimal.Decimal(element[1])
    Gamma_ACE = ((decimal.Decimal(S_rare_ACE)/decimal.Decimal(C_ACE))*\
               (decimal.Decimal(Result_tmp)/(decimal.Decimal(N_rare_ACE)*\
                                             (decimal.Decimal(N_rare_ACE)-1))))-1
    #Finally, determination of S_ACE based on these calculations
    S_ACE = decimal.Decimal(S_abund_ACE) + (decimal.Decimal(S_rare_ACE)/decimal.Decimal(C_ACE))+\
          ((decimal.Decimal(n1_ACE)/decimal.Decimal(C_ACE))*decimal.Decimal(Gamma_ACE))
    Result_tmp = 0
    Results+=str(round(S_ACE,3))+"\t"+str(S_rare_ACE)+"\t"+str(S_abund_ACE)+"\t"

#-------------------------------------------------------------------------------------------------------
#Calculation of bootstrap estimate and also shannon and simpson indexes based on formulas found
#in various publications if wanted (for details : see Appendix.A)
#-------------------------------------------------------------------------------------------------------
#Verification for the choice of the user for calculation of the shannon and simpson indexes
if Index_chose.lower() == "yes":
    S_bootstrap = Nb_clusters
    for element in Matrix:
       S_bootstrap += element[1]*math.pow((1-(element[0]/Nb_seqs_tot)),Nb_seqs_tot)
       x = decimal.Decimal(element[0])/decimal.Decimal(Nb_seqs_tot)
       y = math.log(x)
       z = x*element[1]
       y = decimal.Decimal(str(y))
       H_Shannon += -decimal.Decimal(z*y)
       Result_tmp += decimal.Decimal(z)*decimal.Decimal(str(math.pow(y,2)))
       e = decimal.Decimal(element[1])*decimal.Decimal(element[0])
       h = decimal.Decimal(element[0]) - decimal.Decimal(str(1.0))
       h = decimal.Decimal(h)*decimal.Decimal(e)
       g = decimal.Decimal(Nb_seqs_tot)-decimal.Decimal(str(1.0))
       g = decimal.Decimal(g)*decimal.Decimal(Nb_seqs_tot)
       D_Simpson += decimal.Decimal(h)/decimal.Decimal(g)
    o = decimal.Decimal(Result_tmp)-decimal.Decimal(str(math.pow(H_Shannon,2)))
    o = decimal.Decimal(o) / decimal.Decimal(Nb_seqs_tot)
    p = decimal.Decimal(Nb_clusters) - decimal.Decimal(1)
    q = decimal.Decimal(2)*decimal.Decimal(str(math.pow(Nb_seqs_tot,2)))
    p = decimal.Decimal(p)/decimal.Decimal(q)
    var_H_Shannon = decimal.Decimal(o) + decimal.Decimal(p)
    D_1_Simpson = decimal.Decimal(1)/decimal.Decimal(D_Simpson)
    Result_tmp = 0
    Evenness = decimal.Decimal(H_Shannon)/decimal.Decimal(math.log(decimal.Decimal(Nb_clusters)))
    Results+=str(round(S_bootstrap,3))+"\t"+str(round(H_Shannon,3))\
                    +"\t"+str(var_H_Shannon)+"\t"+str(Evenness)+"\t"+str(D_Simpson)\
                    +"\t"+str(round(D_1_Simpson,3))

#-------------------------------------------------------------------------------------------------------
#Final tasks to print all data acquired for various estimators and indexes.
#-------------------------------------------------------------------------------------------------------
#3.0 : Test to store results in the dedicated file and the dedicated folder.
if ("Concatenated_reads.fasta.fasta" in treated_file):
    Estimators = open("Summary_files/Global_Analysis/"+"Indices_"+ User_choice +"_"+str(cutoff)+".txt", 'a')
else:
    Estimators = open("Summary_files/"+"Indices_"+ User_choice +"_"+str(cutoff)+".txt", 'a')
Estimators.write(Results+"\n")
Estimators.close()
